@@ -49,7 +49,8 @@ sub exists
 
 package Tangram::FlatHash;
 
-use base qw( Tangram::AbstractHash );
+use vars qw(@ISA);
+ @ISA = qw( Tangram::AbstractHash );
 use Tangram::AbstractHash;
 
 $Tangram::Schema::TYPES{flat_hash} = Tangram::FlatHash->new;
@@ -257,8 +258,7 @@ sub erase
 
 	foreach my $def (values %$members)
 	{
-		my $id = $storage->id($obj);
-		$storage->sql_do("DELETE FROM $def->{table} WHERE coll = $id");
+		$storage->sql_do("DELETE FROM $def->{table} WHERE coll = $coll_id");
 	}
 }
 
