@@ -1,10 +1,13 @@
+# (c) Sound Object Logic 2000-2001
+
 use strict;
 
 use Tangram::Scalar;
 
 package Tangram::RawTime;
 
-use base qw( Tangram::String );
+use vars qw(@ISA);
+ @ISA = qw( Tangram::String );
 
 $Tangram::Schema::TYPES{rawtime} = Tangram::RawTime->new;
 
