@@ -5,21 +5,27 @@ use strict;
 package Tangram::Storage;
 use DBI;
 use Carp;
-use Tangram::Core qw(pretty);
+use Tangram::Core;
 
 use vars qw( %storage_class );
 
 BEGIN {
-
-  eval { require 'WeakRef.pm' };
-
-  if ($@) {
-    *Tangram::weaken = sub { };
-    $Tangram::no_weakrefs = 1;
-  } else {
-    *Tangram::weaken = \&WeakRef::weaken;
-    $Tangram::no_weakrefs = 0;
-  }
+    eval 'use Scalar::Util qw()';
+    if ($@) {
+	*Tangram::refaddr = sub { (shift) + 0 };
+	eval 'use WeakRef';
+	if ($@) {
+	    *Tangram::weaken = sub { };
+	    $Tangram::no_weakrefs = 1;
+	} else {
+	    *Tangram::weaken = \&WeakRef::weaken;
+	    $Tangram::no_weakrefs = 0;
+	}
+    } else {
+	*Tangram::weaken = \&Scalar::Util::weaken;
+	*Tangram::refaddr = \&Scalar::Util::refaddr;
+	$Tangram::no_weakrefs = 0;
+    }
 }
 
 sub new
@@ -103,14 +109,14 @@ sub _open
 	  my ($obj, $id) = @_;
 
 	  if ($id) {
-	    $self->{ids}{0 + $obj} = $id;
+	    $self->{ids}{Tangram::refaddr($obj)} = $id;
 	  } else {
-	    delete $self->{ids}{0 + $obj};
+	    delete $self->{ids}{Tangram::refaddr($obj)};
 	  }
 	};
 
     $self->{get_id} = $schema->{get_id} || sub {
-	  my $address = 0 + shift();
+	  my $address = Tangram::refaddr(shift());
 	  my $id = $self->{ids}{$address};
 	  return undef unless $id;
 	  return $id if exists $self->{objects}{$id};
@@ -142,7 +148,15 @@ sub open_connection
     # private - open a new connection to DB for read
 
     my $self = shift;
-    DBI->connect($self->{-cs}, $self->{-user}, $self->{-pw}) or die;
+    my $db = DBI->connect($self->{-cs}, $self->{-user}, $self->{-pw})
+	or die;
+
+    if (exists $self->{no_tx}) {
+	$db->{AutoCommit} = ($self->{no_tx} ? 1 : 0);
+
+    }
+
+    return $db;
 }
 
 sub close_connection
@@ -202,7 +216,9 @@ sub prepare
   {
 	my ($self, $sql) = @_;
 	
-	print $Tangram::TRACE "preparing [@{[ $psi++ ]}] $sql\n" if $Tangram::TRACE;
+	print $Tangram::TRACE "Tangram::Storage: "
+	    ."preparing [@{[ $psi++ ]}] $sql\n"
+	    if $Tangram::TRACE && ($Tangram::DEBUG_LEVEL > 1);
 	$self->{db}->prepare($sql);
   }
 
@@ -268,7 +284,13 @@ sub make_1st_id_in_tx
 	
 	$sth = $self->{make_id}{get};
 	$sth->execute();
-	my $id = $sth->fetchrow_arrayref()->[0];
+    my $row = $sth->fetchrow_arrayref() or
+	die "`Tangram' table corrupt; insert a valid row!";
+	my $id = $row->[0];
+    while ($row =  $sth->fetchrow_arrayref()) {
+	warn "Eep!  More than one row in `Tangram' table!";
+	$id = $row->[0] if ($row->[0] > $id);
+    }
 	$sth->finish();
 
 	return $id;
@@ -367,11 +389,13 @@ sub tx_rollback
 		$self->{db}->rollback if @{ $self->{tx} } == 1; # don't rollback db if nested tx
 		
 		# execute rollback subs in reverse order
-		
-		foreach my $rollback ( @{ pop @{ $self->{tx} } } )
-		  {
-			$rollback->($self);
-		  }
+
+		if (my $rb = pop @{ $self->{tx} }) {
+		    foreach my $rollback ( @$rb )
+			{
+			    $rollback->($self);
+			}
+		}
 	  }
 }
 
@@ -540,7 +564,8 @@ sub _update
 
 	my $sths = $self->{UPDATE_STHS}{$class->{name}} ||=
 	  [ map {
-		print $Tangram::TRACE "preparing $_\n" if $Tangram::TRACE;
+		print $Tangram::TRACE "Tangram::Storage: preparing $_\n"
+		    if ( $Tangram::TRACE && ( $Tangram::DEBUG_LEVEL > 1 ) );
 		$self->prepare($_)
 	  } $engine->get_update_statements($class) ];
 
@@ -702,6 +727,7 @@ sub welcome
   {
     my ($self, $obj, $id) = @_;
     $self->{set_id}->($obj, $id);
+
     Tangram::weaken( $self->{objects}{$id} = $obj );
   }
 
@@ -718,7 +744,7 @@ sub shrink
     my ($self) = @_;
 
     my $objects = $self->{objects};
-    my $prefetch = $self->{prefetch};
+    my $prefetch = $self->{PREFETCH};
 
     for my $id (keys %$objects)
       {
@@ -752,6 +778,9 @@ sub _row_to_object
     my ($self, $obj, $id, $class, $row) = @_;
 	my $context = { storage => $self, id => $id, layout1 => $self->{layout1} };
 	$self->{schema}->classdef($class)->get_importer($context)->($obj, $row, $context);
+    if (my $x=$obj->can("T2_import")) {
+	$x->($obj);
+    }
 	return $obj;
 }
 
@@ -992,18 +1021,31 @@ sub connect
 
 	$opts ||= {};
 
-    my $db = $opts->{dbh} || DBI->connect($cs, $user, $pw);
+    @$self{ -cs, -user, -pw } = ($cs, $user, $pw);
+
+    my $db = $opts->{dbh} || $self->open_connection;
  
-	if ($opts->{no_tx}) {
-	  $self->{no_tx} = 1;
+	if (exists $opts->{no_tx}) {
+	  $self->{no_tx} = $opts->{no_tx};
 	} else {
 	  eval { $db->{AutoCommit} = 0 };
 	  $self->{no_tx} = $db->{AutoCommit};
 	}
 
-    $self->{db} = $db;
+    if (exists $opts->{no_subselects}) {
+	$self->{no_subselects} = $opts->{no_subselects};
+    } else {
+	local($SIG{__WARN__})=sub{};
+	eval {
+	    my $sth = $db->prepare("select * from (select 1+1)");
+	    $sth->execute() or die;
+	};
+	if ($@ or $DBI::errstr) {
+	    $self->{no_subselects} = 1;
+	}
+    }
 
-    @$self{ -cs, -user, -pw } = ($cs, $user, $pw);
+    $self->{db} = $db;
 
     $self->{cid_size} = $schema->{sql}{cid_size};
 	
